// src: https://github.com/garmin/LIDARLite_Arduino_Library/tree/3.0.6
// gps code: https://projecthub.arduino.cc/the_electro_artist/arduino-location-tracker-ed1f85
// must have NEO-6M for the gps

#include <TinyGPS++.h>
#include <SoftwareSerial.h>
// #include <TimerOne.h>
#include <Wire.h>
#include <LIDARLite.h>
#include <SD.h>

LIDARLite myLidarLite;  // LiDar object
TinyGPSPlus gps;        // GPS object
SoftwareSerial ss(RXPin, TXPin); // serial connection to GPS device

struct WaterBody {
  const char* name;
  float latitude;
  float longitude;
};

// if new coordinates or body of water is added then add it here
// define an array of waterbody structures
WaterBody waterCoords = {
  {"mjosa",60.000000,10.000000},
  {"mjosa",60.000000,10.000000},
  {"mjosa",60.000000,10.000000},
  {"mjosa",60.000000,10.000000},
};

// #define DIR_PIN 2
// #define STEP_PIN 3

static const int RXPin = 4, TXPin = 3;
static const uint32_t GPSBaoud = 9600; // baud rate
const float maxRadius = 5; // max distance from location to gps coordinates 


// volatile int stepCount = 0;
// volatile int roundCount = 0;
 
void setup() {
    Serial.begin(GPSBaud); // Initialize serial connection to display distances readings
    myLidarLite.begin(0, true); // Start sensor and I2C
    myLidarLite.configure(0); // set to default mode, balanced performance
    ss.begin(GPSBaud); // Start GPS tracking
 
    // pinMode(DIR_PIN, OUTPUT);
    // pinMode(STEP_PIN, OUTPUT);
 
    // digitalWrite(DIR_PIN, HIGH);
 
    // Timer1.initialize();
    // Timer1.attachInterrupt(tick, 1000000.0f / (3200 * 1));
}

// extra: Reciever bias correction is performed 1 out of every 100 readings
void loop() {

  while (ss.available() > 0){
    gps.encode(ss.read());
    if (gps.location.isUpdated()){ 
      for (int i = 0; i< waterCoors.size(); i++){
        // scan after lidar reached designated location
        if withinRadius(gps.location.lat, gps.location.lng, waterCoordinates[i][1], waterCoordinates[i][2], maxRadius) {
          // Take a measurement with receiver bias correction and print to serial terminal
          receive(true);
        
          // Take 99 measurement without receiver bias correction and print to serial terminal
          for(int i = 0; i < 99; i++){
            receive(false);
          }
        }
      }
    }
  }
}

// Take measurement 
// param bias - with or without reciver bias correction
void receive(bool bias)
{
  Serial.println(myLidarLite.distance(bias));
}

// check if lidar is close enough for a scan
bool withinRadius(float x1,float y1,float x2,float y2, float radius){
  float distance = sqrt(pow((x1-x2),2)+pow((y1-y2),2));
  return (distance <= radius);
}
 
// void tick()
// {
//     digitalWrite(STEP_PIN, HIGH);
//     //delay(1);
//     digitalWrite(STEP_PIN, LOW);
 
//     stepCount++;
//     if(stepCount >= 3200)
//     {
//         stepCount = 0;
//         roundCount++;
//     }
// }